#!/usr/bin/env python
import logging
import sys
from os.path import abspath, dirname, join

import django
from django.conf import settings
from django.test.runner import DiscoverRunner
from edc_constants.constants import CHOL, DM, HIV, HTN
from edc_test_utils import DefaultTestSettings

base_dir = dirname(abspath(__file__))
app_name = "edc_dx"

DEFAULT_SETTINGS = DefaultTestSettings(
    calling_file=__file__,
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=join(base_dir, app_name, "tests", "etc"),
    SUBJECT_SCREENING_MODEL="edc_metadata.subjectscreening",
    SUBJECT_CONSENT_MODEL="edc_metadata.subjectconsent",
    SUBJECT_VISIT_MODEL="edc_metadata.subjectvisit",
    SUBJECT_VISIT_MISSED_MODEL="edc_metadata.subjectvisitmissed",
    SUBJECT_REQUISITION_MODEL="edc_metadata.subjectrequisition",
    LIST_MODEL_APP_LABEL="edc_dx",
    EDC_DX_REVIEW_LIST_MODEL_APP_LABEL="edc_dx_review",
    EDC_DX_REVIEW_SUBJECT_MODEL_APP_LABEL="edc_dx_review",
    EDC_DX_LABELS={
        HIV: HIV,
        DM: "Diabetes",
        HTN: "Hypertension",
        CHOL: "High Cholesterol",
    },
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "multisite",
        "edc_appointment.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_reference.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_dx_review.apps.AppConfig",
        "edc_dx.apps.AppConfig",
    ],
    RANDOMIZATION_LIST_PATH=join(base_dir, app_name, "tests", "test_randomization_list.csv"),
    add_dashboard_middleware=True,
    use_test_urls=True,
).settings


def main():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)
    django.setup()
    tags = [t.split("=")[1] for t in sys.argv if t.startswith("--tag")]
    failures = DiscoverRunner(failfast=False, tags=tags).run_tests([f"{app_name}.tests"])
    sys.exit(failures)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from django.conf import settings
from edc_constants.constants import NO, YES
from edc_model.utils import duration_to_date


class DiagnosisLabelError(Exception):
    pass


try:
    getattr(settings, "EDC_DX_LABELS")
except AttributeError as e:
    raise AttributeError(
        f"{e}. Expected something like `EDC_DX_LABELS=dict"
        "(hiv=HIV,dm=Diabetes,htn=Hypertension,chol=High Cholesterol)`"
    )


def calculate_dx_date_if_estimated(
    dx_date,
    dx_ago,
    report_datetime,
):
    if dx_ago and not dx_date:
        dx_estimated_date = duration_to_date(dx_ago, report_datetime)
        dx_date_is_estimated = YES
    else:
        dx_estimated_date = None
        dx_date_is_estimated = NO
    return dx_estimated_date, dx_date_is_estimated


def get_diagnosis_labels():
    try:
        diagnosis_labels = getattr(settings, "EDC_DX_LABELS")
    except AttributeError as e:
        raise AttributeError(
            f"{e}. Expected something like `EDC_DX_LABELS=dict"
            "(hiv=HIV,dm=Diabetes,htn=Hypertension,chol=High Cholesterol)`"
        )

    return {k.lower(): v for k, v in diagnosis_labels.items()}


def get_diagnosis_labels_prefixes():
    return [k for k in get_diagnosis_labels()]


def raise_on_unknown_diagnosis_labels(obj, fld_suffix, fld_value):
    """Raises an exception if a diagnosis field has a response
    but is not an expected condition.

    See also EDC_DX_LABELS.
    """
    labels = [
        fld.name.split(fld_suffix)[0]
        for fld in obj._meta.get_fields()
        if fld.name.endswith(fld_suffix)
        and fld.name.split(fld_suffix)[0] not in get_diagnosis_labels_prefixes()
        and getattr(obj, fld.name) == fld_value
    ]
    if labels:
        raise DiagnosisLabelError(
            "Diagnosis prefix not expected. See settings.EDC_DX_LABELS. "
            f"Expected one of {get_diagnosis_labels_prefixes()}. Got {labels}."
        )



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Crf, FormsCollection, Schedule, Visit, VisitSchedule

crfs_day1 = FormsCollection(
    Crf(show_order=1, model="edc_dx_review.ClinicalReviewBaseline", required=True),
    Crf(show_order=10, model="edc_dx_review.HivInitialReview", required=False),
    Crf(show_order=20, model="edc_dx_review.HtnInitialReview", required=False),
    Crf(show_order=30, model="edc_dx_review.DmInitialReview", required=False),
    Crf(show_order=40, model="edc_dx_review.CholInitialReview", required=False),
)

crfs = FormsCollection(
    Crf(show_order=1, model="edc_dx_review.ClinicalReview", required=True),
    Crf(show_order=10, model="edc_dx_review.HivInitialReview", required=True),
    Crf(show_order=20, model="edc_dx_review.HtnInitialReview", required=False),
    Crf(show_order=30, model="edc_dx_review.DmInitialReview", required=False),
    Crf(show_order=40, model="edc_dx_review.CholInitialReview", required=False),
    Crf(show_order=50, model="edc_dx_review.HivReview", required=False),
    Crf(show_order=60, model="edc_dx_review.HtnReview", required=False),
    Crf(show_order=70, model="edc_dx_review.DmReview", required=False),
    Crf(show_order=80, model="edc_dx_review.CholReview", required=False),
)

visit0 = Visit(
    code="1000",
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=None,
    crfs=crfs_day1,
    crfs_unscheduled=None,
    requisitions_unscheduled=None,
    facility_name="7-day-clinic",
)

visit1 = Visit(
    code="1010",
    title="Month 1",
    timepoint=1,
    rbase=relativedelta(days=30),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=None,
    crfs=crfs,
    crfs_unscheduled=None,
    requisitions_unscheduled=None,
    facility_name="7-day-clinic",
)


schedule = Schedule(
    name="schedule",
    onschedule_model="edc_metadata.onschedule",
    offschedule_model="edc_metadata.offschedule",
    consent_model="edc_metadata.subjectconsent",
    appointment_model="edc_appointment.appointment",
)

schedule.add_visit(visit0)
schedule.add_visit(visit1)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="edc_metadata.deathreport",
)

visit_schedule.add_schedule(schedule)



from django import forms
from django.test import TestCase
from edc_crf.form_validator_mixins import CrfFormValidatorMixin

from edc_dx import get_diagnosis_labels
from edc_dx.form_validators import DiagnosisFormValidatorMixin

from ..test_case_mixin import TestCaseMixin


class DiagnosisFormValidator(CrfFormValidatorMixin, DiagnosisFormValidatorMixin):
    def clean(self):
        self.get_diagnoses()
        for prefix, label in get_diagnosis_labels():
            self.applicable_if_not_diagnosed(
                prefix=prefix,
                field_applicable=f"{prefix}_test",
                label=label,
            )


class MyModel:
    @classmethod
    def related_visit_model_attr(cls):
        return "subject_visit"


class TestDiagnosisFormValidator(TestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_identifier = self.enroll()
        self.create_visits(self.subject_identifier)

    def test_ok(self):
        data = dict(subject_visit=self.subject_visit_followup)
        form_validator = DiagnosisFormValidator(cleaned_data=data, model=MyModel)
        self.assertRaises(forms.ValidationError, form_validator.validate)
        self.assertIn(
            "Please complete Clinical Review: Baseline",
            str(form_validator._errors.get("__all__")),
        )

    def test_ok2(self):

        data = dict(subject_visit=self.subject_visit_followup)
        form_validator = DiagnosisFormValidator(cleaned_data=data, model=MyModel)
        self.assertRaises(forms.ValidationError, form_validator.validate)
        self.assertIn(
            "Please complete Clinical Review: Baseline",
            str(form_validator._errors.get("__all__")),
        )
